@@ -1,11 +1,11 @@
-// Copyright (c) 2015-2022 The Bitcoin Core developers
+// Copyright (c) 2015-present The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #include <bench/bench.h>
 
 // Extremely fast-running benchmark:
-#include <math.h>
+#include <cmath>
 
 volatile double sum = 0.0; // volatile, global so not optimized away
 
