@@ -7,6 +7,7 @@
 
 #include <consensus/params.h>
 
+#include <array>
 #include <optional>
 #include <string>
 
@@ -14,10 +15,10 @@ struct VBDeploymentInfo {
     /** Deployment name */
     const char *name;
     /** Whether GBT clients can safely ignore this rule in simplified usage */
-    bool gbt_force;
+    bool gbt_optional_rule;
 };
 
-extern const VBDeploymentInfo VersionBitsDeploymentInfo[Consensus::MAX_VERSION_BITS_DEPLOYMENTS];
+extern const std::array<VBDeploymentInfo,Consensus::MAX_VERSION_BITS_DEPLOYMENTS> VersionBitsDeploymentInfo;
 
 std::string DeploymentName(Consensus::BuriedDeployment dep);
 
