@@ -13,10 +13,6 @@
         <source>&amp;New</source>
         <translation type="unfinished">Новы</translation>
     </message>
-    <message>
-        <source>Copy the currently selected address to the system clipboard</source>
-        <translation type="unfinished">Капіяваць пазначаны адрас у сістэмны буфер абмену</translation>
-    </message>
     <message>
         <source>&amp;Copy</source>
         <translation type="unfinished">Капіяваць</translation>
@@ -590,8 +586,19 @@
         <translation type="unfinished">Каталог ужо існуе. Дадайце %1 калі вы збіраецеся стварыць тут новы каталог.</translation>
     </message>
     </context>
+<context>
+    <name>HelpMessageDialog</name>
+    <message>
+        <source>Command-line options</source>
+        <translation type="unfinished">Опцыі каманднага радка</translation>
+    </message>
+</context>
 <context>
     <name>Intro</name>
+    <message>
+        <source>Welcome</source>
+        <translation type="unfinished">Вітаем</translation>
+    </message>
     <message>
         <source>Bitcoin</source>
         <translation type="unfinished">Біткойн</translation>
@@ -633,17 +640,6 @@
         <source>Error</source>
         <translation type="unfinished">Памылка</translation>
     </message>
-    <message>
-        <source>Welcome</source>
-        <translation type="unfinished">Вітаем</translation>
-    </message>
-    </context>
-<context>
-    <name>HelpMessageDialog</name>
-    <message>
-        <source>Command-line options</source>
-        <translation type="unfinished">Опцыі каманднага радка</translation>
-    </message>
 </context>
 <context>
     <name>ModalOverlay</name>
@@ -1093,10 +1089,6 @@
         <source>Error initializing block database</source>
         <translation type="unfinished">Памылка ініцыялізацыі базвы звестак блокаў</translation>
     </message>
-    <message>
-        <source>Error initializing wallet database environment %s!</source>
-        <translation type="unfinished">Памалка ініцыялізацыі асяроддзя базы звестак гаманца %s!</translation>
-    </message>
     <message>
         <source>Error loading block database</source>
         <translation type="unfinished">Памылка загрузкі базвы звестак блокаў</translation>
