@@ -13,10 +13,6 @@
         <source>&amp;New</source>
         <translation type="unfinished">&amp;Jauns</translation>
     </message>
-    <message>
-        <source>Copy the currently selected address to the system clipboard</source>
-        <translation type="unfinished">Kopēt iezīmēto adresi uz starpliktuvi</translation>
-    </message>
     <message>
         <source>&amp;Copy</source>
         <translation type="unfinished">&amp;Kopēt</translation>
@@ -164,6 +160,10 @@
         <source>Enter the old passphrase and new passphrase for the wallet.</source>
         <translation type="unfinished">Ievadiet veco un jauno paroli Jūsu maciņam</translation>
     </message>
+    <message>
+        <source>Continue</source>
+        <translation type="unfinished">Turpināt</translation>
+    </message>
     <message>
         <source>Wallet to be encrypted</source>
         <translation type="unfinished">Maciņu nepieciešams šifrēt.</translation>
@@ -191,6 +191,10 @@
 </context>
 <context>
     <name>QObject</name>
+    <message>
+        <source>Error: %1</source>
+        <translation type="unfinished">Kļūda: %1</translation>
+    </message>
     <message>
         <source>unknown</source>
         <translation type="unfinished">nav zināms</translation>
@@ -206,25 +210,25 @@
     <message numerus="yes">
         <source>%n second(s)</source>
         <translation type="unfinished">
-            <numerusform />
-            <numerusform />
-            <numerusform />
+            <numerusform>%n sekundes</numerusform>
+            <numerusform>%n sekunde</numerusform>
+            <numerusform>%n sekundes</numerusform>
         </translation>
     </message>
     <message numerus="yes">
         <source>%n minute(s)</source>
         <translation type="unfinished">
-            <numerusform />
-            <numerusform />
-            <numerusform />
+            <numerusform>%n minūtes</numerusform>
+            <numerusform>%n minūte</numerusform>
+            <numerusform>%n minūtes</numerusform>
         </translation>
     </message>
     <message numerus="yes">
         <source>%n hour(s)</source>
         <translation type="unfinished">
-            <numerusform />
-            <numerusform />
-            <numerusform />
+            <numerusform>%n stundas</numerusform>
+            <numerusform>%n stunda</numerusform>
+            <numerusform>%n stundas</numerusform>
         </translation>
     </message>
     <message numerus="yes">
@@ -402,10 +406,41 @@
         <source>Up to date</source>
         <translation type="unfinished">Sinhronizēts</translation>
     </message>
+    <message>
+        <source>Open Wallet</source>
+        <translation type="unfinished">Atvērt maku</translation>
+    </message>
+    <message>
+        <source>Close wallet</source>
+        <translation type="unfinished">Aizvērt maku</translation>
+    </message>
+    <message>
+        <source>Close all wallets</source>
+        <translation type="unfinished">Aizvērt visus makus</translation>
+    </message>
+    <message>
+        <source>Wallet Data</source>
+        <extracomment>Name of the wallet data file format.</extracomment>
+        <translation type="unfinished">Maka dati</translation>
+    </message>
+    <message>
+        <source>Restore Wallet</source>
+        <extracomment>Title of pop-up window shown when the user is attempting to restore a wallet.</extracomment>
+        <translation type="unfinished">Atjaunot maku</translation>
+    </message>
+    <message>
+        <source>Wallet Name</source>
+        <extracomment>Label of the input field where the name of the wallet is entered.</extracomment>
+        <translation type="unfinished">Maka nosaukums</translation>
+    </message>
     <message>
         <source>&amp;Window</source>
         <translation type="unfinished">&amp;Logs</translation>
     </message>
+    <message>
+        <source>%1 client</source>
+        <translation type="unfinished">%1 klients</translation>
+    </message>
     <message numerus="yes">
         <source>%n active connection(s) to Bitcoin network.</source>
         <extracomment>A substring of the tooltip.</extracomment>
@@ -415,6 +450,32 @@
             <numerusform />
         </translation>
     </message>
+    <message>
+        <source>Error: %1</source>
+        <translation type="unfinished">Kļūda: %1</translation>
+    </message>
+    <message>
+        <source>Warning: %1</source>
+        <translation type="unfinished">Brīdinājums: %1</translation>
+    </message>
+    <message>
+        <source>Date: %1
+</source>
+        <translation type="unfinished">Datums: %1
+</translation>
+    </message>
+    <message>
+        <source>Wallet: %1
+</source>
+        <translation type="unfinished">Maks: %1
+</translation>
+    </message>
+    <message>
+        <source>Address: %1
+</source>
+        <translation type="unfinished">Adrese: %1
+</translation>
+    </message>
     <message>
         <source>Sent transaction</source>
         <translation type="unfinished">Transakcija nosūtīta</translation>
@@ -491,12 +552,63 @@
         <translation type="unfinished">(bez nosaukuma)</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    <message>
+        <source>Create Wallet</source>
+        <extracomment>Title of window indicating the progress of creation of a new wallet.</extracomment>
+        <translation type="unfinished">Izveidot maku</translation>
+    </message>
+    </context>
+<context>
+    <name>OpenWalletActivity</name>
+    <message>
+        <source>Open Wallet</source>
+        <extracomment>Title of window indicating the progress of opening of a wallet.</extracomment>
+        <translation type="unfinished">Atvērt maku</translation>
+    </message>
+    </context>
+<context>
+    <name>RestoreWalletActivity</name>
+    <message>
+        <source>Restore Wallet</source>
+        <extracomment>Title of progress window which is displayed when wallets are being restored.</extracomment>
+        <translation type="unfinished">Atjaunot maku</translation>
+    </message>
+    </context>
+<context>
+    <name>WalletController</name>
+    <message>
+        <source>Close wallet</source>
+        <translation type="unfinished">Aizvērt maku</translation>
+    </message>
+    <message>
+        <source>Close all wallets</source>
+        <translation type="unfinished">Aizvērt visus makus</translation>
+    </message>
+    </context>
 <context>
     <name>CreateWalletDialog</name>
+    <message>
+        <source>Create Wallet</source>
+        <translation type="unfinished">Izveidot maku</translation>
+    </message>
+    <message>
+        <source>Wallet Name</source>
+        <translation type="unfinished">Maka nosaukums</translation>
+    </message>
     <message>
         <source>Wallet</source>
         <translation type="unfinished">Maciņš</translation>
     </message>
+    <message>
+        <source>Encrypt Wallet</source>
+        <translation type="unfinished">Šifrēt maku</translation>
+    </message>
+    <message>
+        <source>Create</source>
+        <translation type="unfinished">Izveidot</translation>
+    </message>
     </context>
 <context>
     <name>EditAddressDialog</name>
@@ -532,8 +644,38 @@
         <translation type="unfinished">Šeit nevar izveidot datu mapi.</translation>
     </message>
 </context>
+<context>
+    <name>HelpMessageDialog</name>
+    <message>
+        <source>version</source>
+        <translation type="unfinished">versija</translation>
+    </message>
+    <message>
+        <source>Command-line options</source>
+        <translation type="unfinished">Komandrindas iespējas</translation>
+    </message>
+</context>
+<context>
+    <name>ShutdownWindow</name>
+    <message>
+        <source>Do not shut down the computer until this window disappears.</source>
+        <translation type="unfinished">Neizslēdziet datoru kamēr šis logs nepazūd.</translation>
+    </message>
+</context>
 <context>
     <name>Intro</name>
+    <message>
+        <source>Welcome</source>
+        <translation type="unfinished">Sveiciens</translation>
+    </message>
+    <message>
+        <source>Use the default data directory</source>
+        <translation type="unfinished">Izmantot noklusēto datu mapi</translation>
+    </message>
+    <message>
+        <source>Use a custom data directory:</source>
+        <translation type="unfinished">Izmantot pielāgotu datu mapi:</translation>
+    </message>
     <message numerus="yes">
         <source>%n GB of space available</source>
         <translation type="unfinished">
@@ -571,36 +713,6 @@
         <source>Error</source>
         <translation type="unfinished">Kļūda</translation>
     </message>
-    <message>
-        <source>Welcome</source>
-        <translation type="unfinished">Sveiciens</translation>
-    </message>
-    <message>
-        <source>Use the default data directory</source>
-        <translation type="unfinished">Izmantot noklusēto datu mapi</translation>
-    </message>
-    <message>
-        <source>Use a custom data directory:</source>
-        <translation type="unfinished">Izmantot pielāgotu datu mapi:</translation>
-    </message>
-</context>
-<context>
-    <name>HelpMessageDialog</name>
-    <message>
-        <source>version</source>
-        <translation type="unfinished">versija</translation>
-    </message>
-    <message>
-        <source>Command-line options</source>
-        <translation type="unfinished">Komandrindas iespējas</translation>
-    </message>
-</context>
-<context>
-    <name>ShutdownWindow</name>
-    <message>
-        <source>Do not shut down the computer until this window disappears.</source>
-        <translation type="unfinished">Neizslēdziet datoru kamēr šis logs nepazūd.</translation>
-    </message>
 </context>
 <context>
     <name>ModalOverlay</name>
@@ -749,6 +861,14 @@
         <extracomment>Window title text of pop-up box that allows opening up of configuration file.</extracomment>
         <translation type="unfinished">Konfigurāciju Opcijas</translation>
     </message>
+    <message>
+        <source>Continue</source>
+        <translation type="unfinished">Turpināt</translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation type="unfinished">Atcelt</translation>
+    </message>
     <message>
         <source>Error</source>
         <translation type="unfinished">Kļūda</translation>
@@ -796,10 +916,6 @@
         <source>Your current total balance</source>
         <translation type="unfinished">Jūsu kopējā tekošā bilance</translation>
     </message>
-    <message>
-        <source>Spendable:</source>
-        <translation type="unfinished">Iztērējams:</translation>
-    </message>
     <message>
         <source>Recent transactions</source>
         <translation type="unfinished">Nesenās transakcijas</translation>
@@ -819,9 +935,18 @@
         <source>Close</source>
         <translation type="unfinished">Aiztaisīt</translation>
     </message>
+    <message>
+        <source>or</source>
+        <translation type="unfinished">vai</translation>
+    </message>
     </context>
 <context>
     <name>PeerTableModel</name>
+    <message>
+        <source>Received</source>
+        <extracomment>Title of Peers Table column which indicates the total amount of network information we have received from the peer.</extracomment>
+        <translation type="unfinished">Saņemts</translation>
+    </message>
     <message>
         <source>Address</source>
         <extracomment>Title of Peers Table column which contains the IP/Onion/I2P address of the connected peer.</extracomment>
@@ -833,6 +958,14 @@
         <translation type="unfinished">Tīkls</translation>
     </message>
     </context>
+<context>
+    <name>QRImageWidget</name>
+    <message>
+        <source>PNG Image</source>
+        <extracomment>Expanded name of the PNG file format. See: https://en.wikipedia.org/wiki/Portable_Network_Graphics.</extracomment>
+        <translation type="unfinished">PNG attēls</translation>
+    </message>
+</context>
 <context>
     <name>RPCConsole</name>
     <message>
@@ -867,6 +1000,14 @@
         <source>Block chain</source>
         <translation type="unfinished">Bloku virkne</translation>
     </message>
+    <message>
+        <source>Received</source>
+        <translation type="unfinished">Saņemts</translation>
+    </message>
+    <message>
+        <source>Version</source>
+        <translation type="unfinished">Versija</translation>
+    </message>
     <message>
         <source>Last block time</source>
         <translation type="unfinished">Pēdējā bloka laiks</translation>
@@ -903,6 +1044,18 @@
         <source>Out:</source>
         <translation type="unfinished">Iz.:</translation>
     </message>
+    <message>
+        <source>Yes</source>
+        <translation type="unfinished">Jā</translation>
+    </message>
+    <message>
+        <source>No</source>
+        <translation type="unfinished">Nē</translation>
+    </message>
+    <message>
+        <source>From</source>
+        <translation type="unfinished">No</translation>
+    </message>
     </context>
 <context>
     <name>ReceiveCoinsDialog</name>
@@ -984,6 +1137,10 @@
         <source>Label</source>
         <translation type="unfinished">Nosaukums</translation>
     </message>
+    <message>
+        <source>Message</source>
+        <translation type="unfinished">Ziņojums</translation>
+    </message>
     <message>
         <source>(no label)</source>
         <translation type="unfinished">(bez nosaukuma)</translation>
@@ -1067,6 +1224,10 @@
         <source>S&amp;end</source>
         <translation type="unfinished">&amp;Sūtīt</translation>
     </message>
+    <message>
+        <source>or</source>
+        <translation type="unfinished">vai</translation>
+    </message>
     <message>
         <source>Transaction fee</source>
         <translation type="unfinished">Transakcijas maksa</translation>
@@ -1141,10 +1302,6 @@
         <source>Signature</source>
         <translation type="unfinished">Paraksts</translation>
     </message>
-    <message>
-        <source>Copy the current signature to the system clipboard</source>
-        <translation type="unfinished">Kopēt parakstu uz sistēmas starpliktuvi</translation>
-    </message>
     <message>
         <source>Sign the message to prove you own this Bitcoin address</source>
         <translation type="unfinished">Parakstīt ziņojumu lai pierādītu, ka esi šīs Bitcoin adreses īpašnieks.</translation>
@@ -1180,6 +1337,10 @@
         <source>Date</source>
         <translation type="unfinished">Datums</translation>
     </message>
+    <message>
+        <source>From</source>
+        <translation type="unfinished">No</translation>
+    </message>
     <message>
         <source>unknown</source>
         <translation type="unfinished">nav zināms</translation>
@@ -1196,6 +1357,10 @@
         <source>Transaction fee</source>
         <translation type="unfinished">Transakcijas maksa</translation>
     </message>
+    <message>
+        <source>Message</source>
+        <translation type="unfinished">Ziņojums</translation>
+    </message>
     <message>
         <source>Amount</source>
         <translation type="unfinished">Daudzums</translation>
@@ -1225,6 +1390,18 @@
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>This week</source>
+        <translation type="unfinished">Šonedēļ</translation>
+    </message>
+    <message>
+        <source>This month</source>
+        <translation type="unfinished">Šomēnes</translation>
+    </message>
+    <message>
+        <source>This year</source>
+        <translation type="unfinished">Šogad</translation>
+    </message>
     <message>
         <source>Confirmed</source>
         <translation type="unfinished">Apstiprināts</translation>
@@ -1274,7 +1451,16 @@
         <source>Export the data in the current tab to a file</source>
         <translation type="unfinished">Datus no tekošā ieliktņa eksportēt uz failu</translation>
     </message>
-    </context>
+    <message>
+        <source>Wallet Data</source>
+        <extracomment>Name of the wallet data file format.</extracomment>
+        <translation type="unfinished">Maka dati</translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation type="unfinished">Atcelt</translation>
+    </message>
+</context>
 <context>
     <name>bitcoin-core</name>
     <message>
