@@ -14,10 +14,6 @@ Right-click to edit address or label</translation>
         <source>&amp;New</source>
         <translation type="unfinished">&amp;Hou</translation>
     </message>
-    <message>
-        <source>Copy the currently selected address to the system clipboard</source>
-        <translation type="unfinished">Whakakopi te whiriwhiri i te wāhitau kua whiriwhirihia ki te papatohu rorohiko</translation>
-    </message>
     <message>
         <source>&amp;Copy</source>
         <translation type="unfinished">&amp;Kape</translation>
@@ -491,10 +487,6 @@ Ko te whakakī i ēnei whakaaronga e taea ana anake ki ngā whakararuraru o te m
         <source>Invalid port specified in %s: '%s'</source>
         <translation type="unfinished">Kua whakapātaitia te pōti korewhiwhi i roto i %s: '%s'</translation>
     </message>
-    <message>
-        <source>Invalid pre-selected input %s</source>
-        <translation type="unfinished">Kāore i te tika te kōwhiri i tātari i te urupare %s</translation>
-    </message>
     <message>
         <source>Listening for incoming connections failed (listen returned error %s)</source>
         <translation type="unfinished">Whakararuraru ana te whakarongo i ngā hononga e haere mai ana (kua whakahokia te hapa %s)</translation>
@@ -647,14 +639,6 @@ Ko te whakakī i ēnei whakaaronga e taea ana anake ki ngā whakararuraru o te m
         <source>Unable to find UTXO for external input</source>
         <translation type="unfinished">Kāore i te kite i te UTXO mō te whakauru ā-waho</translation>
     </message>
-    <message>
-        <source>Unable to generate initial keys</source>
-        <translation type="unfinished">Kāore e taea te whakaputa i ngā kī tīmatanga</translation>
-    </message>
-    <message>
-        <source>Unable to generate keys</source>
-        <translation type="unfinished">Kāore e taea te whakaputa i ngā kī</translation>
-    </message>
     <message>
         <source>Unable to open %s for writing</source>
         <translation type="unfinished">Kāore e taea te whakatuwhera i %s hei tuhi</translation>
@@ -667,10 +651,6 @@ Ko te whakakī i ēnei whakaaronga e taea ana anake ki ngā whakararuraru o te m
         <source>Unable to start HTTP server. See debug log for details.</source>
         <translation type="unfinished">Kāore e taea te whakahohe i te tūmau HTTP. Tirohia te rārangi whakararuraru mō ngā whakamārama.</translation>
     </message>
-    <message>
-        <source>Unable to unload the wallet before migrating</source>
-        <translation type="unfinished">Kāore e taea te whakakore i te whareparakore i mua i te whakawhiti.</translation>
-    </message>
     <message>
         <source>Unknown -blockfilterindex value %s.</source>
         <translation type="unfinished">-He mea kore te -blockfilterindex whiwhi %s.</translation>
