@@ -1,18 +1,18 @@
-// Copyright (c) 2023 The Bitcoin Core developers
+// Copyright (c) 2023-present The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #include <coins.h>
 #include <crypto/sha256.h>
 #include <primitives/transaction.h>
-#include <test/fuzz/fuzz.h>
 #include <test/fuzz/FuzzedDataProvider.h>
+#include <test/fuzz/fuzz.h>
 #include <test/fuzz/util.h>
 
-#include <assert.h>
-#include <optional>
+#include <cassert>
+#include <cstdint>
 #include <memory>
-#include <stdint.h>
+#include <optional>
 #include <vector>
 
 namespace {
