@@ -30,7 +30,11 @@ from test_framework.messages import (
     MAX_MONEY,
     SEQUENCE_FINAL,
 )
-from test_framework.blocktools import create_tx_with_script, MAX_BLOCK_SIGOPS
+from test_framework.blocktools import (
+    create_tx_with_script,
+    MAX_BLOCK_SIGOPS,
+    MAX_STANDARD_TX_SIGOPS,
+)
 from test_framework.script import (
     CScript,
     OP_0,
@@ -76,7 +80,7 @@ class BadTxTemplate:
     def __init__(self, *, spend_tx=None, spend_block=None):
         self.spend_tx = spend_block.vtx[0] if spend_block else spend_tx
         self.spend_avail = sum(o.nValue for o in self.spend_tx.vout)
-        self.valid_txin = CTxIn(COutPoint(self.spend_tx.sha256, 0), b"", SEQUENCE_FINAL)
+        self.valid_txin = CTxIn(COutPoint(self.spend_tx.txid_int, 0), b"", SEQUENCE_FINAL)
 
     @abc.abstractmethod
     def get_tx(self, *args, **kwargs):
@@ -90,7 +94,6 @@ class OutputMissing(BadTxTemplate):
     def get_tx(self):
         tx = CTransaction()
         tx.vin.append(self.valid_txin)
-        tx.calc_sha256()
         return tx
 
 
@@ -104,7 +107,6 @@ class InputMissing(BadTxTemplate):
     # rather than the input count check.
     def get_tx(self):
         tx = CTransaction()
-        tx.calc_sha256()
         return tx
 
 
@@ -120,7 +122,6 @@ class SizeTooSmall(BadTxTemplate):
         tx.vout.append(CTxOut(0, CScript([OP_RETURN] + ([OP_0] * (MIN_PADDING - 2)))))
         assert len(tx.serialize_without_witness()) == 64
         assert MIN_STANDARD_TX_NONWITNESS_SIZE - 1 == 64
-        tx.calc_sha256()
         return tx
 
 
@@ -128,15 +129,16 @@ class BadInputOutpointIndex(BadTxTemplate):
     # Won't be rejected - nonexistent outpoint index is treated as an orphan since the coins
     # database can't distinguish between spent outpoints and outpoints which never existed.
     reject_reason = None
+    # But fails in block
+    block_reject_reason = "bad-txns-inputs-missingorspent"
 
     def get_tx(self):
         num_indices = len(self.spend_tx.vin)
         bad_idx = num_indices + 100
 
         tx = CTransaction()
-        tx.vin.append(CTxIn(COutPoint(self.spend_tx.sha256, bad_idx), b"", SEQUENCE_FINAL))
+        tx.vin.append(CTxIn(COutPoint(self.spend_tx.txid_int, bad_idx), b"", SEQUENCE_FINAL))
         tx.vout.append(CTxOut(0, basic_p2sh))
-        tx.calc_sha256()
         return tx
 
 
@@ -148,7 +150,6 @@ class DuplicateInput(BadTxTemplate):
         tx.vin.append(self.valid_txin)
         tx.vin.append(self.valid_txin)
         tx.vout.append(CTxOut(1, basic_p2sh))
-        tx.calc_sha256()
         return tx
 
 
@@ -160,19 +161,19 @@ class PrevoutNullInput(BadTxTemplate):
         tx.vin.append(self.valid_txin)
         tx.vin.append(CTxIn(COutPoint(hash=0, n=0xffffffff)))
         tx.vout.append(CTxOut(1, basic_p2sh))
-        tx.calc_sha256()
         return tx
 
 
 class NonexistentInput(BadTxTemplate):
     reject_reason = None  # Added as an orphan tx.
+    # But fails in block
+    block_reject_reason = "bad-txns-inputs-missingorspent"
 
     def get_tx(self):
         tx = CTransaction()
-        tx.vin.append(CTxIn(COutPoint(self.spend_tx.sha256 + 1, 0), b"", SEQUENCE_FINAL))
+        tx.vin.append(CTxIn(COutPoint(self.spend_tx.txid_int + 1, 0), b"", SEQUENCE_FINAL))
         tx.vin.append(self.valid_txin)
         tx.vout.append(CTxOut(1, basic_p2sh))
-        tx.calc_sha256()
         return tx
 
 
@@ -204,13 +205,11 @@ class CreateSumTooLarge(BadTxTemplate):
     def get_tx(self):
         tx = create_tx_with_script(self.spend_tx, 0, amount=MAX_MONEY)
         tx.vout = [tx.vout[0]] * 2
-        tx.calc_sha256()
         return tx
 
 
 class InvalidOPIFConstruction(BadTxTemplate):
     reject_reason = "mempool-script-verify-flag-failed (Invalid OP_IF construction)"
-    valid_in_block = True
 
     def get_tx(self):
         return create_tx_with_script(
@@ -218,7 +217,7 @@ class InvalidOPIFConstruction(BadTxTemplate):
             amount=(self.spend_avail // 2))
 
 
-class TooManySigops(BadTxTemplate):
+class TooManySigopsPerBlock(BadTxTemplate):
     reject_reason = "bad-txns-too-many-sigops"
     block_reject_reason = "bad-blk-sigops, out-of-bounds SigOpCount"
 
@@ -229,6 +228,19 @@ class TooManySigops(BadTxTemplate):
             output_script=lotsa_checksigs,
             amount=1)
 
+
+class TooManySigopsPerTransaction(BadTxTemplate):
+    reject_reason = "bad-txns-too-many-sigops"
+    valid_in_block = True
+
+    def get_tx(self):
+        lotsa_checksigs = CScript([OP_CHECKSIG] * (MAX_STANDARD_TX_SIGOPS + 1))
+        return create_tx_with_script(
+            self.spend_tx, 0,
+            output_script=lotsa_checksigs,
+            amount=1)
+
+
 def getDisabledOpcodeTemplate(opcode):
     """ Creates disabled opcode tx template class"""
     def get_tx(self):
@@ -237,19 +249,18 @@ def getDisabledOpcodeTemplate(opcode):
         vin.scriptSig = CScript([opcode])
         tx.vin.append(vin)
         tx.vout.append(CTxOut(1, basic_p2sh))
-        tx.calc_sha256()
         return tx
 
     return type('DisabledOpcode_' + str(opcode), (BadTxTemplate,), {
         'reject_reason': "disabled opcode",
         'get_tx': get_tx,
-        'valid_in_block' : True
+        'valid_in_block' : False
         })
 
 class NonStandardAndInvalid(BadTxTemplate):
     """A non-standard transaction which is also consensus-invalid should return the first error."""
     reject_reason = "mempool-script-verify-flag-failed (Using OP_CODESEPARATOR in non-witness script)"
-    block_reject_reason = "mandatory-script-verify-flag-failed (OP_RETURN was encountered)"
+    block_reject_reason = "block-script-verify-flag-failed (OP_RETURN was encountered)"
     valid_in_block = False
 
     def get_tx(self):
